@@ -5,12 +5,15 @@ from __future__ import annotations
 
 import logging
 
-import dns
+import dns.query
+import dns.resolver
+import dns.message
 import dns.dnssec
+from dns.rdatatype import RdataType
 
 from expiringdict import ExpiringDict
 
-from checkdmarc.utils import get_base_domain
+from checkdmarc.utils import normalize_domain, get_base_domain
 
 
 """Copyright 2019-2023 Sean Whalen
@@ -34,6 +37,7 @@ TLSA_CACHE = ExpiringDict(max_len=200000, max_age_seconds=1800)
 
 def get_dnskey(
     domain: str,
+    *,
     nameservers: list[str] = None,
     timeout: float = 2.0,
     cache: ExpiringDict = None,
@@ -55,7 +59,7 @@ def get_dnskey(
     if cache is None:
         cache = DNSKEY_CACHE
 
-    domain = domain.lower()
+    domain = normalize_domain(domain)
 
     if domain in cache:
         return cache[domain]
@@ -64,7 +68,7 @@ def get_dnskey(
     request = dns.message.make_query(domain, dns.rdatatype.DNSKEY, want_dnssec=True)
     for nameserver in nameservers:
         try:
-            response = dns.query.udp(request, nameserver, timeout=timeout)
+            response = dns.query.tcp(request, nameserver, timeout=timeout)
             if response is not None:
                 answer = response.answer
                 if len(answer) == 0:
@@ -76,7 +80,11 @@ def get_dnskey(
                         )
                     cache[domain] = None
                     return None
-                rrset = answer[0]
+                rrset = None
+                for rset in answer:
+                    if rset.rdtype != RdataType.RRSIG:
+                        rrset = rset
+                        break
                 name = dns.name.from_text(f"{domain}.")
                 key = {name: rrset}
                 cache[domain] = key
@@ -88,6 +96,7 @@ def get_dnskey(
 
 def test_dnssec(
     domain: str,
+    *,
     nameservers: list[str] = None,
     timeout: float = 2.0,
     cache: ExpiringDict = None,
@@ -126,13 +135,18 @@ def test_dnssec(
         request = dns.message.make_query(domain, rdatatype, want_dnssec=True)
         for nameserver in nameservers:
             try:
-                response = dns.query.udp(request, nameserver, timeout=timeout)
+                response = dns.query.tcp(request, nameserver, timeout=timeout)
                 if response is not None:
                     answer = response.answer
                     if len(answer) != 2:
                         continue
-                    rrset = answer[0]
-                    rrsig = answer[1]
+                    rrset = None
+                    rrsig = None
+                    for rset in answer:
+                        if rset.rdtype == RdataType.RRSIG:
+                            rrsig = rset
+                        else:
+                            rrset = rset
                     dns.dnssec.validate(rrset, rrsig, key)
                     logging.debug(f"Found a signed {rdatatype.name} record")
                     cache[domain] = True
@@ -146,6 +160,7 @@ def test_dnssec(
 
 def get_tlsa_records(
     hostname: str,
+    *,
     nameservers: list[str] = None,
     timeout: float = 2.0,
     port: int = 25,
@@ -182,7 +197,7 @@ def get_tlsa_records(
     )
     for nameserver in nameservers:
         try:
-            response = dns.query.udp(request, nameserver, timeout=timeout)
+            response = dns.query.tcp(request, nameserver, timeout=timeout)
             if response is not None:
                 answer = response.answer
                 if len(answer) != 2:
@@ -196,8 +211,13 @@ def get_tlsa_records(
                         f"a DNSKEY record to verify them"
                     )
                     return tlsa_records
-                rrset = answer[0]
-                rrsig = answer[1]
+                rrset = None
+                rrsig = None
+                for rset in answer:
+                    if rset.rdtype == RdataType.RRSIG:
+                        rrsig = rset
+                    else:
+                        rrset = rset
                 dns.dnssec.validate(rrset, rrsig, dnskey)
                 tlsa_records = list(map(lambda x: str(x), list(rrset.items.keys())))
                 cache[query_hostname] = tlsa_records
