@@ -16,7 +16,7 @@ from pyleri import (
     List,
 )
 
-from checkdmarc.utils import query_dns, WSP_REGEX
+from checkdmarc.utils import normalize_domain, query_dns, WSP_REGEX
 from checkdmarc._constants import SYNTAX_ERROR_MARKER, USER_AGENT, DEFAULT_HTTP_TIMEOUT
 
 """Copyright 2019-2023 Sean Whalen
@@ -141,6 +141,7 @@ STS_TAG_VALUE_REGEX = re.compile(MTA_STS_TAG_VALUE_REGEX_STRING, re.IGNORECASE)
 
 def query_mta_sts_record(
     domain: str,
+    *,
     nameservers: list[str] = None,
     resolver: dns.resolver.Resolver = None,
     timeout: float = 2.0,
@@ -160,13 +161,13 @@ def query_mta_sts_record(
                      - ``record`` - the unparsed MTA-STS record string
                      - ``warnings`` - warning conditions found
 
-     Raises:
+    Raises:
         :exc:`checkdmarc.mta_sts.MTASTSRecordNotFound`
         :exc:`checkdmarc.mta_sts.MTASTSRecordInWrongLocation`
         :exc:`checkdmarc.mta_sts.MultipleMTASTSRecords`
 
     """
-    domain = domain.lower()
+    domain = normalize_domain(domain)
     logging.debug(f"Checking for an MTA-STS record on {domain}")
     warnings = []
     target = f"_mta-sts.{domain}"
@@ -186,7 +187,7 @@ def query_mta_sts_record(
                 unrelated_records.append(record)
 
         if sts_record_count > 1:
-            raise MultipleMTASTSRecords("Multiple MTA-STS records are not permitted")
+            raise MultipleMTASTSRecords("Multiple MTA-STS records are not permitted.")
         if len(unrelated_records) > 0:
             ur_str = "\n\n".join(unrelated_records)
             raise UnrelatedTXTRecordFoundAtMTASTS(
@@ -209,13 +210,12 @@ def query_mta_sts_record(
             for record in records:
                 if record.startswith(txt_prefix):
                     raise MTASTSRecordInWrongLocation(
-                        "The MTA-STS record must be located at "
-                        f"{target}, not {domain}"
+                        f"The MTA-STS record must be located at {target}, not {domain}."
                     )
         except dns.resolver.NoAnswer:
             pass
         except dns.resolver.NXDOMAIN:
-            raise MTASTSRecordNotFound(f"The domain {domain} does not exist")
+            raise MTASTSRecordNotFound(f"The domain {domain} does not exist.")
         except Exception as error:
             raise MTASTSRecordNotFound(error)
     except Exception as error:
@@ -223,7 +223,7 @@ def query_mta_sts_record(
 
     if sts_record is None:
         raise MTASTSRecordNotFound(
-            "An MTA-STS DNS record does not exist for this domain"
+            "An MTA-STS DNS record does not exist for this domain."
         )
 
     return OrderedDict([("record", sts_record), ("warnings", warnings)])
@@ -231,6 +231,7 @@ def query_mta_sts_record(
 
 def parse_mta_sts_record(
     record: str,
+    *,
     include_tag_descriptions: bool = False,
     syntax_error_marker: str = SYNTAX_ERROR_MARKER,
 ) -> OrderedDict:
@@ -256,10 +257,10 @@ def parse_mta_sts_record(
             ``include_tag_descriptions`` is set to ``True``
 
     Raises:
-        :exc:`checkdmarc.mta_sts.MTASTSSyntaxError`
+        :exc:`checkdmarc.mta_sts.MTASTSRecordSyntaxError`
         :exc:`checkdmarc.mta_sts.InvalidMTASTSTag`
-        :exc:`checkdmarc.mta_sts.InvalidMTASTSTagValue`
-        :exc:`checkdmarc.mta_sts.SPFRecordFoundWhereSTSRecordShouldBe`
+        :exc:`checkdmarc.mta_sts.InvalidSTSTagValue`
+        :exc:`checkdmarc.mta_sts.SPFRecordFoundWhereMTASTSRecordShouldBe`
 
     """
     logging.debug("Parsing the MTA-STS record")
@@ -300,7 +301,7 @@ def parse_mta_sts_record(
         tag = pair[0].lower().strip()
         tag_value = str(pair[1].strip())
         if tag not in mta_sts_tags:
-            raise InvalidMTASTSTag(f"{tag} is not a valid MTA-STS record tag")
+            raise InvalidMTASTSTag(f"{tag} is not a valid MTA-STS record tag.")
         tags[tag] = OrderedDict(value=tag_value)
         if include_tag_descriptions:
             tags[tag]["description"] = mta_sts_tags[tag]["description"]
@@ -309,7 +310,7 @@ def parse_mta_sts_record(
 
 
 def download_mta_sts_policy(
-    domain: str, http_timeout: float = DEFAULT_HTTP_TIMEOUT
+    domain: str, *, http_timeout: float = DEFAULT_HTTP_TIMEOUT
 ) -> OrderedDict:
     """
     Downloads a domains MTA-HTS policy
@@ -380,7 +381,7 @@ def parse_mta_sts_policy(policy: str) -> OrderedDict:
     acceptable_keys = required_keys.copy()
     acceptable_keys.append("mx")
     if "\n" in policy and "\r\n" not in policy:
-        warnings.append("MTA-STS policy lines should end with CRLF not LF")
+        warnings.append("MTA-STS policy lines should end with CRLF not LF.")
         policy = policy.replace("\n", "\r\n")
     lines = policy.split("\r\n")
     for i in range(len(lines)):
@@ -389,7 +390,7 @@ def parse_mta_sts_policy(policy: str) -> OrderedDict:
             continue
         key_value = lines[i].split(":")
         if len(key_value) != 2:
-            raise MTASTSPolicySyntaxError(f"Line {line}: Not a key: value pair")
+            raise MTASTSPolicySyntaxError(f"Line {line}: Not a key: value pair.")
         key = key_value[0].strip()
         value = key_value[1].strip()
         if key not in acceptable_keys:
@@ -401,7 +402,7 @@ def parse_mta_sts_policy(policy: str) -> OrderedDict:
         elif key == "mode" and value not in modes:
             MTASTSPolicySyntaxError(f"Line {line}: Invalid mode: {value}")
         elif key == "max_age":
-            error_msg = "max_age must be an integer value between 0 and " "31557600"
+            error_msg = "max_age must be an integer value between 0 and 31557600."
             if "." in value:
                 raise MTASTSPolicySyntaxError(error_msg)
             try:
@@ -414,17 +415,15 @@ def parse_mta_sts_policy(policy: str) -> OrderedDict:
             parsed_policy[key] = value
         else:
             if len(MTA_STS_MX_REGEX.findall(value)) == 0:
-                raise MTASTSPolicySyntaxError(
-                    f"Line {line}: Invalid mx " f"value: {value}"
-                )
+                raise MTASTSPolicySyntaxError(f"Line {line}: Invalid mx value: {value}")
             mx.append(value)
     for required_key in required_keys:
         if required_key not in parsed_policy:
-            raise MTASTSPolicySyntaxError(f"Missing required key: " f"{required_key}")
+            raise MTASTSPolicySyntaxError(f"Missing required key: {required_key}.")
 
     if parsed_policy["mode"] != "none" and len(mx) == 0:
         raise MTASTSPolicySyntaxError(
-            f"{parsed_policy['mode']} mode requires " f"at least one mx value"
+            f"{parsed_policy['mode']} mode requires at least one mx value."
         )
     parsed_policy["mx"] = mx
 
@@ -433,6 +432,7 @@ def parse_mta_sts_policy(policy: str) -> OrderedDict:
 
 def check_mta_sts(
     domain: str,
+    *,
     nameservers: list[str] = None,
     resolver: dns.resolver.Resolver = None,
     timeout: float = 2.0,
@@ -461,7 +461,7 @@ def check_mta_sts(
                       - ``error`` - Tne error message
                       - ``valid`` - False
     """
-    domain = domain.lower()
+    domain = normalize_domain(domain)
     mta_sts_results = OrderedDict([("valid", True)])
     try:
         mta_sts_record = query_mta_sts_record(
@@ -470,7 +470,7 @@ def check_mta_sts(
         warnings = mta_sts_record["warnings"]
         mta_sts_record = parse_mta_sts_record(mta_sts_record["record"])
         mta_sts_results["id"] = mta_sts_record["tags"]["id"]["value"]
-        policy = download_mta_sts_policy(domain, timeout=timeout)
+        policy = download_mta_sts_policy(domain, http_timeout=timeout)
         warnings += policy["warnings"]
         policy = parse_mta_sts_policy(policy["policy"])
         warnings += policy["warnings"]
